from django.apps import AppConfig


class WagtailAbTestingTestAppConfig(AppConfig):
    label = "wagtail_ab_testing_test"
    name = "wagtail_ab_testing.test"
    verbose_name = "Wagtail A/B Testing tests"
